@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2020 Leandro Nini
+ * Copyright 2011-2021 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -103,14 +103,16 @@ void IniConfig::clear()
     console_s.junctionRight = '+';
 
     audio_s.frequency = SidConfig::DEFAULT_SAMPLING_FREQ;
-    audio_s.playback  = SidConfig::MONO;
+    audio_s.channels  = 0;
     audio_s.precision = 16;
 
     emulation_s.modelDefault  = SidConfig::PAL;
     emulation_s.modelForced   = false;
     emulation_s.sidModel      = SidConfig::MOS6581;
     emulation_s.forceModel    = false;
+#ifdef FEAT_CONFIG_CIAMODEL
     emulation_s.ciaModel      = SidConfig::MOS6526;
+#endif
     emulation_s.filter        = true;
     emulation_s.engine.clear();
 
@@ -246,7 +248,9 @@ bool readTime(iniHandler &ini, const TCHAR *key, int &value)
         return false;
 
     int time;
+    int milliseconds = 0;
     const size_t sep = str.find_first_of(':');
+    const size_t dot = str.find_first_of('.');
     try
     {
         if (sep == SID_STRING::npos)
@@ -254,13 +258,31 @@ bool readTime(iniHandler &ini, const TCHAR *key, int &value)
             time = dataParser::parseInt(str.c_str());
         }
         else
-        {   // Read in MM:SS format
+        {   // Read in MM:SS.mmm format
             const int min = dataParser::parseInt(str.substr(0, sep).c_str());
             if (min < 0 || min > 99)
                 goto IniCofig_readTime_error;
             time = min * 60;
 
-            const int sec  = dataParser::parseInt(str.substr(sep + 1).c_str());
+            int sec;
+            if (dot == SID_STRING::npos)
+            {
+                sec  = dataParser::parseInt(str.substr(sep + 1).c_str());
+            }
+            else
+            {
+                sec  = dataParser::parseInt(str.substr(sep + 1, dot - sep).c_str());
+                SID_STRING msec = str.substr(dot + 1);
+                milliseconds = dataParser::parseInt(msec.c_str());
+                switch (msec.length())
+                {
+                case 1: milliseconds *= 100; break;
+                case 2: milliseconds *= 10; break;
+                case 3: break;
+                default: goto IniCofig_readTime_error;
+                }
+            }
+
             if (sec < 0 || sec > 59)
                 goto IniCofig_readTime_error;
             time += sec;
@@ -272,7 +294,7 @@ bool readTime(iniHandler &ini, const TCHAR *key, int &value)
         return false;
     }
 
-    value = time;
+    value = time * 1000 + milliseconds;
     return true;
 
 IniCofig_readTime_error:
@@ -337,20 +359,9 @@ void IniConfig::readAudio(iniHandler &ini)
     if (!ini.setSection (TEXT("Audio")))
         ini.addSection(TEXT("Audio"));
 
-    {
-        int frequency = (int) audio_s.frequency;
-        readInt(ini, TEXT("Frequency"), frequency);
-        audio_s.frequency = (unsigned long) frequency;
-    }
+    readInt(ini, TEXT("Frequency"), audio_s.frequency);
 
-    {
-        int channels = 0;
-        readInt(ini, TEXT("Channels"),  channels);
-        if (channels)
-        {
-            audio_s.playback = (channels == 1) ? SidConfig::MONO : SidConfig::STEREO;
-        }
-    }
+    readInt(ini, TEXT("Channels"),  audio_s.channels);
 
     readInt(ini, TEXT("BitsPerSample"), audio_s.precision);
 }
@@ -380,7 +391,7 @@ void IniConfig::readEmulation(iniHandler &ini)
 
     readBool(ini, TEXT("ForceC64Model"), emulation_s.modelForced);
     readBool(ini, TEXT("DigiBoost"), emulation_s.digiboost);
-
+#ifdef FEAT_CONFIG_CIAMODEL
     {
         SID_STRING str = readString(ini, TEXT("CiaModel"));
         if (!str.empty())
@@ -391,7 +402,7 @@ void IniConfig::readEmulation(iniHandler &ini)
                 emulation_s.ciaModel = SidConfig::MOS8521;
         }
     }
-
+#endif
     {
         SID_STRING str = readString(ini, TEXT("SidModel"));
         if (!str.empty())
@@ -494,7 +505,7 @@ bool tryOpen(iniHandler &ini)
         // Try exec dir first
         SID_STRING execPath(utils::getExecPath());
         execPath.append(SEPARATOR).append(FILE_NAME);
-        if (ini.open(execPath.c_str()))
+        if (ini.tryOpen(execPath.c_str()))
             return true;
     }
 #endif
