@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2019 Leandro Nini
+ * Copyright 2011-2021 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,8 @@
 
 #include "ini/types.h"
 
+#include "sidlib_features.h"
+
 #include "sidcxx11.h"
 
 using std::cout;
@@ -84,6 +86,7 @@ bool parseTime(const char *str, uint_least32_t &time)
         return false;
 
     uint_least32_t _time;
+    uint_least32_t milliseconds = 0;
 
     char *sep = (char *) strstr (str, ":");
     if (!sep)
@@ -91,21 +94,40 @@ bool parseTime(const char *str, uint_least32_t &time)
         _time = atoi (str);
     }
     else
-    {   // Read in MM:SS format
-        // TODO parse milliseconds too
+    {   // Read in MM:SS[.mmm] format
         int val;
         *sep = '\0';
         val  = atoi (str);
         if (val < 0 || val > 99)
             return false;
         _time = (uint_least32_t) val * 60;
+        // parse milliseconds
+        char *milli = (char *) strstr (sep+1, ".");
+        if (milli)
+        {
+            char *start = milli + 1;
+            char *end;
+            milliseconds = strtol(start, &end, 10);
+            switch (end - start)
+            {
+            case 1: milliseconds *= 100; break;
+            case 2: milliseconds *= 10; break;
+            case 3: break;
+            default: return false;
+            }
+
+            if (milliseconds < 0 || milliseconds > 999)
+                return false;
+
+            *milli = '\0';
+        }
         val   = atoi (sep + 1);
         if (val < 0 || val > 59)
             return false;
         _time += (uint_least32_t) val;
     }
 
-    time = _time * 1000;
+    time = _time * 1000 + milliseconds;
     return true;
 }
 
@@ -126,7 +148,7 @@ void displayDebugArgs()
 
     out << "Debug Options:" << endl
         << " --cpu-debug   display cpu register and assembly dumps" << endl
-        << " --delay=<num> simulate c64 power on delay" << endl
+        << " --delay=<num> simulate c64 power on delay (default: random)" << endl
         << " --noaudio     no audio output device" << endl
         << " --nosid       no sid emulation" << endl
         << " --none        no audio output device and no sid emulation" << endl;
@@ -182,7 +204,7 @@ int ConsolePlayer::args(int argc, const char *argv[])
                 if (!parseAddress (&argv[i][3], m_engCfg.secondSidAddress))
                     err = true;
             }
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_THIRD_SID
             else if (strncmp (&argv[i][1], "ts", 2) == 0)
             {   // Override sidTune and enable the third sid
                 if (!parseAddress (&argv[i][3], m_engCfg.thirdSidAddress))
@@ -263,7 +285,12 @@ int ConsolePlayer::args(int argc, const char *argv[])
 
             else if (argv[i][1] == 's')
             {   // Stereo Playback
-                m_engCfg.playback = SidConfig::STEREO;
+                m_channels = 2;
+            }
+
+            else if (argv[i][1] == 'm')
+            {   // Mono Playback
+                m_channels = 1;
             }
 
             else if (argv[i][1] == 't')
@@ -312,11 +339,12 @@ int ConsolePlayer::args(int argc, const char *argv[])
             {
                 m_engCfg.defaultSidModel = SidConfig::MOS8580;
             }
+#ifdef FEAT_DIGIBOOST
             else if (strcmp (&argv[i][1], "-digiboost") == 0)
             {
                 m_engCfg.digiBoost = true;
             }
-
+#endif
             // Video/Verbose Options
             else if (strcmp (&argv[i][1], "vnf") == 0)
             {
@@ -526,24 +554,28 @@ int ConsolePlayer::args(int argc, const char *argv[])
             if (!dbOpened)
             {
                 // Try load user configured songlength DB
+                if ((m_iniCfg.sidplay2()).database.length() != 0)
+                {
+                    // Try loading the database specificed by the user
 #if defined(_WIN32) && defined(UNICODE)
-                // FIXME
-                char database[MAX_PATH];
-                const int ret = wcstombs(database, (m_iniCfg.sidplay2()).database.c_str(), sizeof(database));
-                if (ret >= MAX_PATH)
-                    database[0] = '\0';
+#  ifdef FEAT_DB_WCHAR_OPEN
+                    const wchar_t *database = (m_iniCfg.sidplay2()).database.c_str();
+#  else
+                    char database[MAX_PATH];
+                    const int ret = wcstombs(database, (m_iniCfg.sidplay2()).database.c_str(), sizeof(database));
+                    if (ret >= MAX_PATH)
+                        database[0] = '\0';
+#  endif
 #else
-                const char *database = (m_iniCfg.sidplay2()).database.c_str();
+                    const char *database = (m_iniCfg.sidplay2()).database.c_str();
 #endif
-                if (strlen(database) != 0)
-                {   // Try loading the database specificed by the user
                     if (!m_database.open(database))
                     {
                         displayError (m_database.error ());
                         return -1;
                     }
 
-                    if (strstr(database, ".md5") != nullptr)
+                    if ((m_iniCfg.sidplay2()).database.find(TEXT(".md5")) != SID_STRING::npos)
                         newSonglengthDB = true;
                 }
             }
@@ -581,12 +613,12 @@ void ConsolePlayer::displayArgs (const char *arg)
     out << "Options:" << endl
         << " --help|-h    display this screen" << endl
         << " --help-debug debug help menu" << endl
-        << " -b<num>      set start time in [m:]s format (default 0)" << endl
+        << " -b<num>      set start time in [m:]s format (default: 0)" << endl
 
         << " -f<num>      set frequency in Hz (default: "
         << SidConfig::DEFAULT_SAMPLING_FREQ << ")" << endl
         << " -ds<addr>    set second sid address (e.g. -ds0xd420)" << endl
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_THIRD_SID
         << " -ts<addr>    set third sid address (e.g. -ts0xd440)" << endl
 #endif
 
@@ -600,20 +632,20 @@ void ConsolePlayer::displayArgs (const char *arg)
         << " -p<num>      set format for file output (16 = signed 16 bit, 32 = 32 bit float)"
         << "(default: 16)" << endl
 
-#if !defined(DISALLOW_STEREO_SOUND)
-        << " -s           stereo sid support" << endl
-#endif
+        << " -s           force stereo output" << endl
+        << " -m           force mono output" << endl
 
         << " -t<num>      set play length in [m:]s format (0 is endless)" << endl
 
-        << " -<v|q>       verbose or quiet (no time display) output" << endl
+        << " -<v|q>[x]    verbose or quiet output. x is the optional level, default=1" << endl
         << " -v[p|n][f]   set VIC PAL/NTSC clock speed (default: defined by song)" << endl
         << "              Use 'f' to force the clock by preventing speed fixing" << endl
 
         << " -m<o|n>[f]   set SID new/old chip model (default: old)" << endl
         << "              Use 'f' to force the model" << endl
+#ifdef FEAT_DIGIBOOST
         << " --digiboost  Enable digiboost for 8580 model" << endl
-
+#endif
         << " -r[i|r][f]   set resampling method (default: resample interpolate)" << endl
         << "              Use 'f' to enable fast resampling (only for reSID)" << endl
 
