@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2019 Leandro Nini
+ * Copyright 2011-2021 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,8 @@
 
 #include "player.h"
 
+#include "codeConvert.h"
+
 #include <cstring>
 #include <ctype.h>
 
@@ -41,6 +43,20 @@ using std::string;
 #include <sidplayfp/SidInfo.h>
 #include <sidplayfp/SidTuneInfo.h>
 
+#ifdef FEAT_REGS_DUMP_SID
+const char *noteName[] =
+{
+    "C-0", "C#0", "D-0", "D#0", "E-0", "F-0", "F#0", "G-0", "G#0", "A-0", "A#0", "B-0",
+    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1", "G-1", "G#1", "A-1", "A#1", "B-1",
+    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2", "G-2", "G#2", "A-2", "A#2", "B-2",
+    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3", "G-3", "G#3", "A-3", "A#3", "B-3",
+    "C-4", "C#4", "D-4", "D#4", "E-4", "F-4", "F#4", "G-4", "G#4", "A-4", "A#4", "B-4",
+    "C-5", "C#5", "D-5", "D#5", "E-5", "F-5", "F#5", "G-5", "G#5", "A-5", "A#5", "B-5",
+    "C-6", "C#6", "D-6", "D#6", "E-6", "F-6", "F#6", "G-6", "G#6", "A-6", "A#6", "B-6",
+    "C-7", "C#7", "D-7", "D#7", "E-7", "F-7", "F#7", "G-7", "G#7", "A-7", "A#7", "B-7",
+};
+#endif
+
 const char SID6581[] = "MOS6581";
 const char SID8580[] = "CSG8580";
 
@@ -87,8 +103,25 @@ const char* getClock(SidTuneInfo::clock_t clock)
         return "ANY";
     }
 }
+#ifdef FEAT_REGS_DUMP_SID
+const char *ConsolePlayer::getNote(uint16_t freq)
+{
+    if (freq)
+    {
+        int distance = 0xffff;
+        for (int i=0; i<(12 * 8); i++)
+        {
+            int d = abs(freq - m_freqTable[i]);
+            if (d < distance)
+                distance = d;
+            else
+                return noteName[i];
+        }
+    }
 
-
+    return "---";
+}
+#endif
 // Display console menu
 void ConsolePlayer::menu ()
 {
@@ -126,25 +159,27 @@ void ConsolePlayer::menu ()
     const unsigned int n = tuneInfo->numberOfInfoStrings();
     if (n)
     {
+        codeConvert codeset;
+
         consoleTable (tableSeparator);
 
         consoleTable  (tableMiddle);
         consoleColour (cyan, true);
         cerr << " Title        : ";
         consoleColour (magenta, true);
-        cerr << tuneInfo->infoString(0) << endl;
+        cerr << codeset.convert(tuneInfo->infoString(0)) << endl;
         if (n>1)
         {
             consoleTable  (tableMiddle);
             consoleColour (cyan, true);
             cerr << " Author       : ";
             consoleColour (magenta, true);
-            cerr << tuneInfo->infoString(1) << endl;
+            cerr << codeset.convert(tuneInfo->infoString(1)) << endl;
             consoleTable  (tableMiddle);
             consoleColour (cyan, true);
             cerr << " Released     : ";
             consoleColour (magenta, true);
-            cerr << tuneInfo->infoString(2) << endl;
+            cerr << codeset.convert(tuneInfo->infoString(2)) << endl;
         }
     }
 
@@ -240,6 +275,9 @@ void ConsolePlayer::menu ()
         const uint_least32_t seconds = m_timer.stop / 1000;
         cerr << setw(2) << setfill('0') << ((seconds / 60) % 100)
              << ':' << setw(2) << setfill('0') << (seconds % 60);
+#ifdef FEAT_NEW_SONLEGTH_DB
+        cerr << '.' << setw(3) << m_timer.stop % 1000;
+#endif
     }
     else if (m_timer.valid)
         cerr << "FOREVER";
@@ -294,13 +332,13 @@ void ConsolePlayer::menu ()
         cerr << " SID Details  : ";
         consoleColour (white, false);
         cerr << "Model = ";
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_NEW_TUNEINFO_API
         cerr << getModel(tuneInfo->sidModel(0));
 #else
         cerr << getModel(tuneInfo->sidModel1());
 #endif
         cerr << endl;
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_NEW_TUNEINFO_API
         if (tuneInfo->sidChips() > 1)
 #else
         if (tuneInfo->isStereo())
@@ -310,7 +348,7 @@ void ConsolePlayer::menu ()
             consoleColour (yellow, true);
             cerr << "              : ";
             consoleColour (white, false);
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_NEW_TUNEINFO_API
             cerr << "2nd SID = $" << hex << tuneInfo->sidChipBase(1) << dec;
             cerr << ", Model = " << getModel(tuneInfo->sidModel(1));
 #else
@@ -318,7 +356,7 @@ void ConsolePlayer::menu ()
             cerr << ", Model = " << getModel(tuneInfo->sidModel2());
 #endif
             cerr << endl;
-#if LIBSIDPLAYFP_VERSION_MAJ > 1 || LIBSIDPLAYFP_VERSION_MIN >= 8
+#ifdef FEAT_NEW_TUNEINFO_API
             if (tuneInfo->sidChips() > 2)
             {
                 consoleTable  (tableMiddle);
@@ -351,13 +389,13 @@ void ConsolePlayer::menu ()
         cerr << " SID Filter   : ";
         consoleColour (white, false);
         cerr << (m_filter.enabled ? "Yes" : "No") << endl;
-
+#ifdef FEAT_DIGIBOOST
         consoleTable  (tableMiddle);
         consoleColour (yellow, true);
         cerr << " DigiBoost    : ";
         consoleColour (white, false);
         cerr << (m_engCfg.digiBoost ? "Yes" : "No") << endl;
-
+#endif
         consoleTable  (tableMiddle);
         consoleColour (yellow, true);
         cerr << " SID Model    : ";
@@ -431,6 +469,19 @@ void ConsolePlayer::menu ()
     }
     cerr << endl;
 
+#ifdef FEAT_REGS_DUMP_SID
+    if (m_verboseLevel > 1)
+    {
+        consoleTable  (tableSeparator);
+        consoleTable (tableMiddle); cerr << "         NOTE PW         CONTROL          WAVEFORMS" << endl;
+
+        for (int i=0; i < tuneInfo->sidChips() * 3; i++)
+        {
+            consoleTable (tableMiddle); cerr << endl; // reserve space for the Voice 3 status
+        }
+    }
+#endif
+
     consoleTable (tableEnd);
 
     if (m_driver.file)
@@ -445,6 +496,106 @@ void ConsolePlayer::menu ()
     cerr << flush;
 }
 
+void ConsolePlayer::refreshRegDump()
+{
+#ifdef FEAT_REGS_DUMP_SID
+    if (m_verboseLevel > 1)
+    {
+        const SidTuneInfo *tuneInfo = m_tune.getInfo();
+
+        cerr << "\x1b[" << tuneInfo->sidChips() * 3 + 1 << "A\r"; // Moves cursor X lines up
+
+        for (int j=0; j < tuneInfo->sidChips(); j++)
+        {
+            uint8_t* registers = m_registers[j];
+            uint8_t oldCtl[3];
+            oldCtl[0] = registers[0x04];
+            oldCtl[1] = registers[0x0b];
+            oldCtl[2] = registers[0x12];
+
+            if (m_engine.getSidStatus(j, registers))
+            {
+                oldCtl[0] ^= registers[0x04];
+                oldCtl[1] ^= registers[0x0b];
+                oldCtl[2] ^= registers[0x12];
+
+                for (int i=0; i < 3; i++)
+                {
+                    consoleTable (tableMiddle);
+                    consoleColour (red, true);
+
+                    cerr << " Voice " << (j * 3 + i+1) << hex;
+
+                    consoleColour (yellow, true);
+                    ;
+                    cerr << " " << getNote(registers[0x00 + i * 0x07] | ((registers[0x01 + i * 0x07] & 0x0f) << 8))
+                         << " $" << setw(3) << setfill('0') << (registers[0x02 + i * 0x07] | ((registers[0x03 + i * 0x07] & 0x0f) << 8));
+
+                    // gate changed ?
+                    consoleColour((oldCtl[i] & 0x01) ? green : red, true);
+                    // gate on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x01) ? " GATE" : " gate");
+
+                    // sync changed ?
+                    consoleColour((oldCtl[i] & 0x02) ? green : red, true);
+                    // sync on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x02) ? " SYNC" : " sync");
+
+                    // ring changed ?
+                    consoleColour((oldCtl[i] & 0x04) ? green : red, true);
+                    // ring on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x04) ? " RING" : " ring");
+
+                    // test changed ?
+                    consoleColour((oldCtl[i] & 0x08) ? green : red, true);
+                    // test on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x08) ? " TEST" : " test");
+
+                    // triangle changed ?
+                    consoleColour((oldCtl[i] & 0x10) ? green : red, true);
+                    // triangle on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x10) ? " TRI" : " ___");
+
+                    // sawtooth changed ?
+                    consoleColour((oldCtl[i] & 0x20) ? green : red, true);
+                    // sawtooth on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x20) ? " SAW" : " ___");
+
+                    // pulse changed ?
+                    consoleColour((oldCtl[i] & 0x40) ? green : red, true);
+                    // pulse on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x40) ? " PUL" : " ___");
+
+                    // noise changed ?
+                    consoleColour((oldCtl[i] & 0x80) ? green : red, true);
+                    // noise on ?
+                    cerr << ((registers[0x04 + i * 0x07] & 0x80) ? " NOI" : " ___");
+
+                    cerr << dec << endl;
+                }
+            }
+            else
+            {
+                consoleTable (tableMiddle); cerr << "???" << endl;
+                consoleTable (tableMiddle); cerr << "???" << endl;
+                consoleTable (tableMiddle); cerr << "???" << endl;
+            }
+        }
+
+        consoleTable (tableEnd);
+    }
+    else
+#endif
+        cerr << "\r";
+
+    if (m_driver.file)
+        cerr << "Creating audio file, please wait...";
+    else
+        cerr << "Playing, press ESC to stop...";
+
+    cerr << flush;
+}
+
 // Set colour of text on console
 void ConsolePlayer::consoleColour (player_colour_t colour, bool bold)
 {
